#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <dos.h>

#include "ksys.h"


static unsigned numbits(unsigned long d)
{
    unsigned aux = 0;
    do
    {
        aux++;
        d >>= 1;
    } while (d);
    return aux-1;
}

static unsigned bitset(int i, unsigned long d)
{
    int aux = ((d & (1L << i)) > 0);
    return aux;
}

static unsigned long esm1(unsigned long C, unsigned long d, unsigned long n)
{
    register i, k;
    double aux;

    k = numbits(d);
    aux = C;
    for(i=k-1;i>=0;i--)
    {
        aux = fmod(aux * aux, n);
        if (bitset(i,d) == 1) aux = fmod(aux * C, n);
    }
    return aux;
}


int     checkregistration(char *fname, KEY *ki,
                        unsigned long publickey,
                        unsigned long commonkey)
{
    int result = -1, i;
    FILE *fp;
    KEY unenc;
    unsigned long enc[sizeof(KEY) / sizeof(unsigned)];

    if (!(fp = fopen(fname,"rb")))
        return result;
    if (1 != fread(&unenc, sizeof(unenc), 1, fp))
        return result;
    if (1 != fread(enc, sizeof(enc), 1, fp))
        return result;
    fclose(fp);

    if (unenc.publickey != publickey)
        return result;
    if (unenc.commonkey != commonkey)
        return result;

    for(i=0;i<64 / sizeof(unsigned);i++)
    {
        *((unsigned *)ki + i) = esm1(enc[i], publickey, commonkey);
    }

    if (memcmp(ki, &unenc, sizeof(unenc)))
        return result;

    result = 0;
    return result;
}

int     createkeyfile(char *fname, char *whoto,
                    unsigned regnumber,
                    int      kind,
                    unsigned long privatekey,
                    unsigned long publickey,
                    unsigned long commonkey)
{
    int result = -1, i;
    struct date dt;
    FILE *fp;
    KEY ki, unenc;
    unsigned long enc[sizeof(KEY) / sizeof(unsigned)];

    getdate(&dt);
    memset(&unenc, 0, sizeof(unenc));
    strncpy(unenc.whoto, whoto, sizeof(unenc.whoto) - 1);
    kind = toupper(kind);
    if (!strchr("DNB", kind))
        kind = 'D';
    *unenc.cnumber = kind;
    sprintf((char *)&unenc.cnumber[1], "%05u", regnumber);
    unenc.number = regnumber;
    unenc.commonkey = commonkey;
    unenc.publickey = publickey;
    unenc.day = dt.da_day;
    unenc.month = dt.da_mon;
    unenc.year = dt.da_year;

    for(i=0;i<64 / sizeof(unsigned);i++)
    {
        enc[i] = esm1(*((unsigned *)&unenc + i), privatekey, commonkey);
    }

    for(i=0;i<64 / sizeof(unsigned);i++)
    {
        *((unsigned *)&ki + i) = esm1(enc[i], publickey, commonkey);
    }

    if (memcmp(&ki, &unenc, sizeof(unenc)))
        return result;

    if (!(fp = fopen(fname,"wb")))
        return result;
    if (1 != fwrite(&unenc, sizeof(unenc), 1, fp))
        return result;
    if (1 != fwrite(enc, sizeof(enc), 1, fp))
        return result;
    fclose(fp);

    result = 0;
    return result;
}
